#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/9/26 11:28
# @Author  : dmj-11740
# @File    : OCR.py
# @Software: PyCharm
# @desc    :
import base64
import json
from io import BytesIO

import requests
from PIL import Image


class OCRConfig:
    api = "http://192.168.125.115:3344"


class OCR:

    @staticmethod
    def ocr_file(file_path):
        """
            ocr识别
        :param file_path: 文件路径
        :return:
        """
        api_url = f"{OCRConfig.api}/ocr/file/json"
        file = open(file_path, "rb").read()
        resp = requests.post(api_url, files={"image": file})
        return resp.json()

    @staticmethod
    def ocr_base64(base64_str):
        """
            ocr识别
        :param base64_str: 图片base64
        :return:
        """
        api_url = f"{OCRConfig.api}/ocr/b64/json"
        resp = requests.post(api_url, data=base64_str)
        return resp.json()

    @staticmethod
    def det_file(file_path):
        """
            目标识别
        :param file_path: 文件路径
        :return:
        """
        api_url = f"{OCRConfig.api}/det/file/json"
        file = open(file_path, "rb").read()
        resp = requests.post(api_url, files={"image": file})
        return resp.json()

    @staticmethod
    def det_base64(base64_str):
        """
            目标识别
        :param base64_str: 图片base64
        :return:
        """
        api_url = f"{OCRConfig.api}/det/b64/json"
        resp = requests.post(api_url, data=base64_str)
        return resp.json()

    @staticmethod
    def slide_file(target_file,
                   bg_file,
                   target_to_size: tuple[int, int] = None,
                   bg_to_size: tuple[int, int] = None,
                   algorithm=1):
        """
            滑块识别
        :param target_file: 目标图片路径
        :param bg_file: 背景图片路径
        :param target_to_size: 目标图片需要大小
        :param bg_to_size: 背景图片需要大小
        :param algorithm: 算法，1为匹配:滑块图和坑位图匹配，2为比较:坑位图和背景图比较
        :return:
        """
        method = "match"
        if algorithm != 1:
            method = "compare"
        target_file = open(target_file, "rb").read()
        target_file_bytes = OCR.content_to_bytes(target_file, target_to_size)
        bg_file = open(bg_file, "rb").read()
        bg_file_bytes = OCR.content_to_bytes(bg_file, bg_to_size)
        api_url = f"{OCRConfig.api}/slide/{method}/file/json"
        resp = requests.post(
            api_url, files={"target_img": target_file_bytes, "bg_img": bg_file_bytes}
        )
        return resp.json()

    @staticmethod
    def slide_base64(target_b64str,
                     bg_b64str,
                     target_to_size: tuple[int, int] = None,
                     bg_to_size: tuple[int, int] = None,
                     algorithm=1):
        """
            滑块识别
        :param target_b64str: 目标图片路径
        :param bg_b64str: 背景图片路径
        :param target_to_size: 目标图片需要大小
        :param bg_to_size: 背景图片需要大小
        :param algorithm: 算法，1为匹配:滑块图和坑位图匹配，2为比较:坑位图和背景图比较
        :return:
        """
        method = "match"
        if algorithm != 1:
            method = "compare"
        target_b64str = OCR.content_to_bytes(target_b64str, target_to_size)
        target_b64str = base64.b64encode(target_b64str).decode()
        bg_b64str = OCR.content_to_bytes(bg_b64str, bg_to_size)
        bg_b64str = base64.b64encode(bg_b64str).decode()
        json_str = json.dumps({"target_img": target_b64str, "bg_img": bg_b64str})
        api_url = f"{OCRConfig.api}/slide/{method}/b64/json"
        resp = requests.post(api_url, data=base64.b64encode(json_str.encode()).decode())
        return resp.json()

    @staticmethod
    def content_to_bytes(image_data, to_size: tuple[int, int] = None):
        if isinstance(image_data, bytes):
            image = Image.open(BytesIO(image_data))
        else:
            if "," in image_data:
                image_data = image_data.split(",")[1]
            image_data = base64.b64decode(image_data)
            image = Image.open(BytesIO(image_data))
        # 8位图片转换为32位图片，否则ddddocr无法处理
        if image.mode == "P":
            image = image.convert("RGBA")
        img_byte_arr = BytesIO()
        if to_size:
            image = image.resize(to_size)
        image.save(img_byte_arr, format="PNG")

        return img_byte_arr.getvalue()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Time    : 2025/7/11 10:20
@Author  : dmj-11740
@File    : tess.py
@Software: PyCharm
@desc    : 
"""
import base64
import os
import random
import time
from copy import deepcopy
from io import BytesIO

import ddddocr
import cv2
from PIL import Image
import numpy as np
from PIL import Image, ImageDraw, ImageFont
from numpy.linalg import norm
from scipy.optimize import linear_sum_assignment


class SliderCS:

    def __init__(self):
        self.det = ddddocr.DdddOcr(det=True)
        self.ocr = ddddocr.DdddOcr()

    def expand_rect(self, rect, expand_pixel=10):
        """
        扩大矩形坐标框

        参数:
        rect (list): 原始矩形坐标 [x1, y1, x2, y2]
        expand_pixel (int): 向四周扩大的像素值（默认10像素）

        返回:
        list: 扩大后的矩形坐标 [new_x1, new_y1, new_x2, new_y2]
        """
        x1, y1, x2, y2 = rect
        # 计算扩大后的坐标
        new_x1 = x1 - expand_pixel
        new_y1 = y1 - expand_pixel
        new_x2 = x2 + expand_pixel
        new_y2 = y2 + expand_pixel
        # 确保坐标不为负数（可选，根据实际场景）
        new_x1 = max(0, new_x1)
        new_y1 = max(0, new_y1)
        return [new_x1, new_y1, new_x2, new_y2]

    def crop_image_by_coordinates(self, image, coords):
        """
        根据坐标列表切割图片
        coords (list): 坐标列表，格式为[x1, y1, x2, y2]
        """
        try:
            coords = self.expand_rect(coords, 3)
            x1, y1, x2, y2 = coords
            if image is None:
                print(f"错误：无法读取图片 {image}")
                return False
            height, width = image.shape[:2]
            if x1 < 0 or y1 < 0 or x2 > width or y2 > height or x1 >= x2 or y1 >= y2:
                print(
                    f"错误：无效的坐标范围。图片尺寸: {width}x{height}，给定坐标: ({x1},{y1}) 到 ({x2},{y2})"
                )
                return False
            cropped_image = image[y1:y2, x1:x2]

            # print(f"成功切割图片并保存到 {output_path}")
            retval, buffer = cv2.imencode(".png", cropped_image)
            # 将缓冲区转换为二进制数据
            binary_data = buffer.tobytes()

            return binary_data

        except Exception as e:
            print(f"切割图片时发生错误: {str(e)}")
            return False

    def convert_background(self, image_data, ts=False):
        if isinstance(image_data, bytes):
            image = Image.open(BytesIO(image_data))
        else:
            if image_data.startswith("data:image/"):
                image_data = image_data.split(",")[1]
            image_data = base64.b64decode(image_data)
            image = Image.open(BytesIO(image_data))
        with open(f"tt-{'hint' if ts else 'bg'}.png", "wb") as f:
            f.write(image_data)
        cv_rgb_image = np.array(image)
        image = cv2.cvtColor(cv_rgb_image, cv2.COLOR_RGB2BGR)
        height, width = image.shape[:2]
        if ts:
            image = image[0:height, 300:width]
        else:
            image = cv2.resize(image, (290, 179), interpolation=cv2.INTER_AREA)

        retval, buffer = cv2.imencode(".jpg", image)

        # 将缓冲区转换为二进制数据
        binary_data = buffer.tobytes()

        bboxes = self.det.detection(binary_data)
        bboxes = sorted(bboxes, key=lambda x: x[0])

        print(bboxes)
        A = []
        img = Image.open(BytesIO(binary_data))
        for idx, bbox in enumerate(bboxes):
            result = self.crop_image_by_coordinates(image, bbox)
            result = self.ocr.classification(result)
            if not result:
                for angle in range(-20, 20, 5):
                    result = self.img_rotate_discern(img, angle)
                    if result:
                        break
            if not result:
                result = '人'
            A.append(result[0])
        return bboxes, A

    def char_to_vector(self, ch, size=32, font_path="C:/Windows/Fonts/simhei.ttf"):
        img = Image.new("L", (size, size), color=255)  # 白底
        draw = ImageDraw.Draw(img)
        font = ImageFont.truetype(font_path, size=size - 4)

        bbox = draw.textbbox((0, 0), ch, font=font)
        w, h = bbox[2] - bbox[0], bbox[3] - bbox[1]
        draw.text(((size - w) / 2, (size - h) / 2), ch, fill=0, font=font)

        vec = np.array(img).astype(float).flatten()
        vec = (vec - vec.mean()) / (vec.std() + 1e-5)
        return vec

    def cosine_similarity(self, vec1, vec2):
        return np.dot(vec1, vec2) / (norm(vec1) * norm(vec2))

    def match_indices_unique(
            self, list1, list2, font_path="C:/Windows/Fonts/simhei.ttf"
    ):
        # 向量化
        vec1s = [self.char_to_vector(ch, font_path=font_path) for ch in list1]
        vec2s = [self.char_to_vector(ch, font_path=font_path) for ch in list2]

        # 相似度矩阵 (list1 行, list2 列)
        sim_matrix = np.zeros((len(list1), len(list2)))
        for i, v1 in enumerate(vec1s):
            for j, v2 in enumerate(vec2s):
                sim_matrix[i, j] = self.cosine_similarity(v1, v2)

        # 匈牙利算法：因为 linear_sum_assignment 是最小化代价，所以取 -sim
        row_ind, col_ind = linear_sum_assignment(-sim_matrix)

        # 输出结果
        result = {}
        for i, j in zip(row_ind, col_ind):
            # result[list1[i]] = (j, sim_matrix[i, j])  # (索引, 相似度)
            result[list1[i]] = j  # 索引
        return result

    def img_rotate_discern(self, image, angle):
        rotated_img = image.rotate(
            angle, expand=True, resample=Image.Resampling.BICUBIC
        )
        result = self.ocr.classification(rotated_img)
        # rotated_img.show()
        return result

    def get_center_coordinate(self, rect):
        """
        计算矩形区域的中间坐标（中心点）

        参数:
        rect (list): 矩形坐标，格式为 [x1, y1, x2, y2]

        返回:
        tuple: 中心点坐标 (center_x, center_y)
        """
        x1, y1, x2, y2 = rect
        center_x = (x1 + x2) / 2
        center_y = (y1 + y2) / 2
        return (center_x, center_y)


def get_select_coord(hint, background):
    """
        点选验证坐标识别
    :param hint:
    :param background:
    :return:
    """
    scs = SliderCS()
    if os.path.exists(hint):
        with open(hint, "rb") as f:
            hint = f.read()
    if os.path.exists(background):
        with open(background, "rb") as f:
            background = f.read()
    bboxes1, list1 = scs.convert_background(hint, True)
    bboxes2, list2 = scs.convert_background(background, False)
    list1_b = deepcopy(list1)
    bboxes2_b = deepcopy(bboxes2)
    for word1 in list1:
        if word1 in list2:
            list1_b.remove(word1)
            idx = list2.index(word1)
            bboxes2_b.remove(bboxes2[idx])

    # print(list1, list1_b)
    # print(list2)
    if background.startswith("data:image/"):
        background = background.split(",")[1]
    image_data = base64.b64decode(background)
    image = Image.open(BytesIO(image_data))
    cv_rgb_image = np.array(image)
    image = cv2.cvtColor(cv_rgb_image, cv2.COLOR_RGB2BGR)
    for boxe in bboxes2_b:
        if not list1_b:
            break
        binary_data = scs.crop_image_by_coordinates(image, boxe)
        img = Image.open(BytesIO(binary_data))
        for angle in range(-20, 20, 5):
            result = scs.img_rotate_discern(img, angle)
            if result in list1_b:
                print("结果", result)
                idx = bboxes2.index(boxe)
                list2[idx] = result
                list1_b.remove(result)
                break
    print("最后优化识别词：", list1)
    print("最后优化识别选：", list2)
    coors = scs.match_indices_unique(
        list1, list2, font_path="C:/Windows/Fonts/simhei.ttf"
    )
    print({coor: list2[idx] for coor, idx in coors.items()})
    result = [(scs.get_center_coordinate(bboxes2[coor])) for coor in coors.values()]
    # print(result)
    return result



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/10/14 9:59
# @Author  : dmj-11740
# @File    : slider.py
# @Software: PyCharm
# @desc    : 滑块轨迹生成
import io

import numpy as np
import math
import random
import base64

import ddddocr
from PIL import Image


def __ease_out_expo(sep):
    if sep == 1:
        return 1
    else:
        return 1 - pow(2, -10 * sep)

def get_target(self, img_bytes: bytes = None):
    image = Image.open(io.BytesIO(img_bytes))
    return image.crop([1, 1, 47, 47]), 1, 1


def get_slide_track(distance):
    """
        V1:根据滑动距离生成滑动轨迹
    :param distance: 需要滑动的距离
    :return: 滑动轨迹<type 'list'>: [[x,y,t], ...]
        x: 已滑动的横向距离
        y: 已滑动的纵向距离, 除起点外, 均为0
        t: 滑动过程消耗的时间, 单位: 毫秒
    """

    if not isinstance(distance, int) or distance < 0:
        raise ValueError(
            f"distance类型必须是大于等于0的整数: distance: {distance}, type: {type(distance)}"
        )
    # 初始化轨迹列表
    slide_track = [
        [random.randint(-50, -10), random.randint(-50, -10), 0],
        [0, 0, 0],
    ]
    # 共记录count次滑块位置信息
    count = 30 + int(distance / 2)
    # 初始化滑动时间
    t = random.randint(50, 100)
    # 记录上一次滑动的距离
    _x = 0
    _y = 0
    for i in range(count):
        # 已滑动的横向距离
        x = round(__ease_out_expo(i / count) * distance)
        # 滑动过程消耗的时间
        t += random.randint(10, 20)
        if x == _x:
            continue
        slide_track.append([x, _y, t])
        _x = x
    slide_track.append(slide_track[-1])
    return slide_track


def generate_trajectory(distance, shake: bool = True, shake_space: int = 5) -> list:
    """
        根据滑动距离生成自动化操作所用的动作链
    :param distance: 滑动距离
    :param shake: 滑动距离是否左右抖动
    :param shake_space: 抖动范围
    :return:滑动轨迹<type 'list'>: [[x,y,t], ...]
        x: 本次滑动距离
        y: 本次上下抖动距离
        t: 滑动过程消耗的时间, 单位: 秒
    """
    if shake:
        distance += random.randint(-shake_space, shake_space)
    slide_track = get_slide_track(distance)
    trajectory = []
    last_x = 0
    last_time = 0
    for x, y, time in slide_track:
        this_time = time - last_time
        this_x = x - last_x
        trajectory.append([this_x, y, this_time])
        last_time = time
        last_x = x
    # 将轨迹列表中的时间转换为毫秒
    trajectory = [[x, y, time / 1000] for x, y, time in trajectory if x > 0]
    return trajectory


class SliderTrajectory:
    """
        贝赛尔曲线轨迹生成
    """

    def _bztsg(self, dataTrajectory):
        lengthOfdata = len(dataTrajectory)

        def staer(x):
            t = ((x - dataTrajectory[0][0]) / (dataTrajectory[-1][0] - dataTrajectory[0][0]))
            y = np.array([0, 0], dtype=np.float64)
            for s in range(len(dataTrajectory)):
                y += dataTrajectory[s] * ((math.factorial(lengthOfdata - 1) / (
                        math.factorial(s) * math.factorial(lengthOfdata - 1 - s))) * math.pow(t, s) * math.pow(
                    (1 - t), lengthOfdata - 1 - s))
            return y[1]

        return staer

    def _type(self, type, x, numberList):
        numberListre = []
        pin = (x[1] - x[0]) / numberList
        if type == 0:
            for i in range(numberList):
                numberListre.append(i * pin)
            if pin >= 0:
                numberListre = numberListre[::-1]
        elif type == 1:
            for i in range(numberList):
                numberListre.append(1 * ((i * pin) ** 2))
            numberListre = numberListre[::-1]
        elif type == 2:
            for i in range(numberList):
                numberListre.append(1 * ((i * pin - x[1]) ** 2))

        elif type == 3:
            dataTrajectory = [np.array([0, 0]), np.array([(x[1] - x[0]) * 0.8, (x[1] - x[0]) * 0.6]),
                              np.array([x[1] - x[0], 0])]
            fun = self._bztsg(dataTrajectory)
            numberListre = [0]
            for i in range(1, numberList):
                numberListre.append(fun(i * pin) + numberListre[-1])
            if pin >= 0:
                numberListre = numberListre[::-1]
        numberListre = np.abs(np.array(numberListre) - max(numberListre))
        biaoNumberList = ((numberListre - numberListre[numberListre.argmin()]) / (
                numberListre[numberListre.argmax()] - numberListre[numberListre.argmin()])) * (x[1] - x[0]) + x[0]
        biaoNumberList[0] = x[0]
        biaoNumberList[-1] = x[1]
        return biaoNumberList

    def _getFun(self, s):
        '''

        :param s: 传入P点
        :return: 返回公式
        '''
        dataTrajectory = []
        for i in s:
            dataTrajectory.append(np.array(i))
        return self._bztsg(dataTrajectory)

    def _simulation(self, start, end, le=1, deviation=0, bias=0.5):
        '''

        :param start:开始点的坐标 如 start = [0, 0]
        :param end:结束点的坐标 如 end = [100, 100]
        :param le:几阶贝塞尔曲线，越大越复杂 如 le = 4
        :param deviation:轨迹上下波动的范围 如 deviation = 10
        :param bias:波动范围的分布位置 如 bias = 0.5
        :return:返回一个字典equation对应该曲线的方程，P对应贝塞尔曲线的影响点
        '''
        start = np.array(start)
        end = np.array(end)
        cbb = []
        if le != 1:
            e = (1 - bias) / (le - 1)
            cbb = [[bias + e * i, bias + e * (i + 1)] for i in range(le - 1)]

        dataTrajectoryList = [start]

        t = random.choice([-1, 1])
        w = 0
        for i in cbb:
            px1 = start[0] + (end[0] - start[0]) * (random.random() * (i[1] - i[0]) + (i[0]))
            p = np.array([px1, self._bztsg([start, end])(px1) + t * deviation])
            dataTrajectoryList.append(p)
            w += 1
            if w >= 2:
                w = 0
                t = -1 * t

        dataTrajectoryList.append(end)
        # print(dataTrajectoryList)
        return {"equation": self._bztsg(dataTrajectoryList), "P": np.array(dataTrajectoryList)}

    def trackArray(self, start, end, numberList, le=1, deviation=0, bias=0.5, type=0, cbb=0, yhh=10):
        '''

        :param start:开始点的坐标 如 start = [0, 0]
        :param end:结束点的坐标 如 end = [100, 100]
        :param numberList:返回的数组的轨迹点的数量 numberList = 150
        :param le:几阶贝塞尔曲线，越大越复杂 如 le = 4
        :param deviation:轨迹上下波动的范围 如 deviation = 10
        :param bias:波动范围的分布位置 如 bias = 0.5
        :param type:0表示均速滑动，1表示先慢后快，2表示先快后慢，3表示先慢中间快后慢 如 type = 1
        :param cbb:在终点来回摆动的次数
        :param yhh:在终点来回摆动的范围
        :return:返回一个字典trackArray对应轨迹数组，P对应贝塞尔曲线的影响点
        '''
        s = []
        fun = self._simulation(start, end, le, deviation, bias)
        w = fun['P']
        fun = fun["equation"]
        if cbb != 0:
            numberListOfcbb = round(numberList * 0.2 / (cbb + 1))
            numberList -= (numberListOfcbb * (cbb + 1))

            xTrackArray = self._type(type, [start[0], end[0]], numberList)
            for i in xTrackArray:
                s.append([i, fun(i)])
            dq = yhh / cbb
            kg = 0
            ends = np.copy(end)
            for i in range(cbb):
                if kg == 0:
                    d = np.array([end[0] + (yhh - dq * i),
                                  ((end[1] - start[1]) / (end[0] - start[0])) * (end[0] + (yhh - dq * i)) + (
                                          end[1] - ((end[1] - start[1]) / (end[0] - start[0])) * end[0])])
                    kg = 1
                else:
                    d = np.array([end[0] - (yhh - dq * i),
                                  ((end[1] - start[1]) / (end[0] - start[0])) * (end[0] - (yhh - dq * i)) + (
                                          end[1] - ((end[1] - start[1]) / (end[0] - start[0])) * end[0])])
                    kg = 0
                # print(d)
                y = self.trackArray(ends, d, numberListOfcbb, le=2, deviation=0, bias=0.5, type=0, cbb=0, yhh=10)
                s += list(y['trackArray'])
                ends = d
            y = self.trackArray(ends, end, numberListOfcbb, le=2, deviation=0, bias=0.5, type=0, cbb=0, yhh=10)
            s += list(y['trackArray'])

        else:
            xTrackArray = self._type(type, [start[0], end[0]], numberList)
            for i in xTrackArray:
                s.append([i, fun(i)])

        return {"trackArray": np.array(s), "P": w}

    def generate(self, distance, action_count: int = 0, action_time: int = 0):
        """
            根据滑动距离生成滑动轨迹
        :param distance: 需要滑动的距离
        :param action_count: 生成操作数量
        :param action_time: 操作总用时
        :return: 滑动轨迹<type 'list'>: [[x,y,t], ...]
            x: 滑动的横向距离
            y: 滑动的纵向距离
            t: 滑动过程消耗的时间, 单位: 秒
        """
        if action_time == 0:
            action_time = random.randint(700, 2000)
        if action_count == 0:
            action_count = distance // 2 + random.randint(10, 20)
        interval_time = action_time // action_count
        le = random.randint(2, 4)
        action_speed_type = random.randint(0, 3)
        data = self.trackArray([0, 0], np.array([distance, 0]), action_count, le, 20, type=action_speed_type, cbb=2)
        slider_track = [[int(v[0]), int(v[1]), interval_time / 1000] for v in data['trackArray']]
        return slider_track


class Slider:
    def __init__(self, resize_data: list):
        """
            拼图滑块识别
        :param resize_data: 图片在网页中实际大小[(小图x,小图y),(大图x,大图y)]
        """
        self.tailor_y = None
        self.resize_data = resize_data

    def content_to_bytes(self, image_data, image_type):
        first_row_with_pixel, end_row_with_pixel = None, None
        if isinstance(image_data, bytes):
            image = Image.open(io.BytesIO(image_data))
        else:
            if image_data.startswith("data:image/"):
                image_data = image_data.split(",")[1]
            image_data = base64.b64decode(image_data)
            image = Image.open(io.BytesIO(image_data))
        # 8位图片转换为32位图片，否则ddddocr无法处理
        if image.mode == "P":
            image = image.convert("RGBA")
        if image.mode == "GBA":
            image = image.convert("RGBA")
        # 前端显示的大小非图片大小
        if image_type == "background":
            image = image.resize(self.resize_data[0])
            if self.tailor_y:
                box = (0, self.tailor_y[0], 290, self.tailor_y[1])
                image = image.crop(box)

        elif image_type == "slider":
            image = image.resize(self.resize_data[1])
            pixel = image.load()
            for y in range(image.size[1]):
                row_not_pixel = True
                for x in range(image.size[0]):
                    r, g, b, a = pixel[x, y]
                    if a != 0 and not first_row_with_pixel:
                        first_row_with_pixel = y
                    if a != 0:
                        row_not_pixel = False
                if first_row_with_pixel and row_not_pixel:
                    end_row_with_pixel = y
                if first_row_with_pixel and end_row_with_pixel:
                    break
            first_row_with_pixel = first_row_with_pixel - 10
            end_row_with_pixel = end_row_with_pixel + 10
            self.tailor_y = (first_row_with_pixel, end_row_with_pixel)
            image = image.crop((0, first_row_with_pixel, 60, end_row_with_pixel))

        img_byte_arr = io.BytesIO()
        image.save(img_byte_arr, format="PNG")

        return img_byte_arr.getvalue()

    def slider_discern(self, slider, background):
        """
            拼图滑块识别
        :param slider: 小图base64
        :param background: 大图base64
        :return: 滑动距离
        """
        try:
            ddddocr.DdddOcr.get_target = get_target
            slide = ddddocr.DdddOcr(det=False, ocr=False)
            slider = self.content_to_bytes(slider, "slider")
            background = self.content_to_bytes(background, "background")
            res = slide.slide_match(slider, background)
            print(f"滑块识别结果[可能需要修正边框距离]：{res}")
            return res["target"][0]
        except Exception as e:
            print(f"滑块识别失败:{e}")
            return 100





from .client import DangquSdk
from .OCR import OCR, OCRConfig
from .excel_read import ExcelRead, BList
from .slider import get_slide_track, generate_trajectory, SliderTrajectory, Slider
from .click_select import get_select_coord

__all__ = [
    'DangquSdk',
    "OCR",
    'OCRConfig',
    "Slider",
    'ExcelRead',
    'BList',
    'SliderTrajectory',
    'generate_trajectory',
    'get_slide_track',
    'get_select_coord'
]



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Time    : 2025/2/27 13:34
@Author  : dmj-11740
@File    : excel_read.py
@Software: PyCharm
@desc    : 
"""
import json
import os
import warnings


class BList(list):

    def to_excel(
        self,
        file_name,
        sheet_name: str = "sheet1",
        mode: str = "w",
    ):
        try:
            import pandas
        except:
            raise Exception("pandas not found!")
        data = []
        have_convert = False
        for key, value in self[0].items():
            if isinstance(value, dict) or isinstance(value, list):
                have_convert = True
                break
        if have_convert:
            for item in self:
                for key, value in item.items():
                    if isinstance(value, dict) or isinstance(value, list):
                        item[key] = json.dumps(value, ensure_ascii=False)
                data.append(item)
        pad = pandas.DataFrame(data or self)
        mode = "w" if not os.path.exists(file_name + ".xlsx") else mode
        with pandas.ExcelWriter(
            file_name + ".xlsx",
            mode=mode,
            engine="xlsxwriter",
            # options={"strings_to_urls": False},
            engine_kwargs={"options": {"strings_to_urls": False}},
        ) as writer:
            pad.to_excel(writer, index=False, sheet_name=sheet_name)


class ExcelRead:
    def __init__(self, file_path, sheet_name=0, header=0, **kwargs):
        """
            读取Excel
        :param file_path:  文件路径
        :param sheet_name: 表名
        :param kwargs:
        """
        self.path = file_path
        self.sheet_name = sheet_name
        self.header = header
        self._sheet_names = None
        self.__values = None
        self.__sheet_name_bck = None

    @property
    def values(self) -> BList:
        # 读取Excel文件
        try:
            import pandas
        except:
            raise Exception("pandas not found!")
        if self.__values:
            return self.__values
        if not os.path.exists(self.path):
            raise FileNotFoundError(f"找不到文件:{self.path}")
        # excel_name = self.path.rsplit("/", 1)[1] if "/" in self.path else self.path
        # loading = Loading(f"文件[{excel_name}]")
        with warnings.catch_warnings(record=True):
            warnings.simplefilter("ignore", ResourceWarning)
            df = pandas.read_excel(
                self.path, sheet_name=self.sheet_name, dtype="str", header=self.header
            )
        # 替换Excel表格内的空单元格，否则在下一步处理中将会报错
        df.fillna("", inplace=True)
        df_list = []
        for row in df.index.values:
            # loc为按列名索引 iloc为按位置索引，使用的是 [[行号], [列名]]
            df_line = df.loc[row, df.columns.values.tolist()].to_dict()
            # print(df.loc[row, df.columns.values.tolist()])
            for k, v in df_line.items():
                if v == "true" or v == "True":
                    df_line[k] = True
                elif v == "false" or v == "False":
                    df_line[k] = False
                # if not isinstance(v, str):
                #     df_line[k] = str(v)
            df_list.append(df_line)
        # loading.stop()
        self.__values = BList(df_list)
        return self.__values

    @property
    def values_str(self) -> list:
        json_str = []
        for value_json in self.values:
            json_str.append(
                {key: value.__str__() for key, value in value_json.mow_list()}
            )
        return json_str

    def only_index(self, index: str, *args) -> dict:
        if not self.__values:
            self.__values = self.values
        index = [index]
        index.extend(args)
        item = {}
        for val in self.__values:
            keys = "###".join([val.get(k, "") for k in index])
            assert (
                keys
            ), f"有值为空:{val[list(val.keys())[0]]}, {val[list(val.keys())[1]]}"
            if keys not in item:
                item[keys] = []
            item[keys].append(val)
        return item

    @property
    def values_all(self) -> BList:
        try:
            import pandas
        except:
            raise Exception("pandas not found!")
        excel_file = pandas.ExcelFile(self.path)
        self._sheet_names = excel_file.sheet_names
        items = BList()
        self.__sheet_name_bck = self.sheet_name
        for sheet_name in self._sheet_names:
            self.sheet_name = sheet_name
            item = self.values
            items.extend(item)
        self.sheet_name = self.__sheet_name_bck
        return items




from setuptools import setup, find_packages

setup(
    name='DangQu',
    version='0.1.3',
    description='dangqu sdk',
    author='DMJ-11740',
    author_email='rpa@lumi.com',
    packages=find_packages(),
    install_requires=[
        'requests',
        'oauthlib',
        'requests-oauthlib',
        'pillow',
        'numpy',
        'ddddocr',
        'scipy',
        'opencv-python'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)
